# 以下はテスト
import json

from tokenizer import parse_indented_text
from parser import parser
from converter import convert
from renderer import Renderer

text = """\
page text:サンプルページ
    grid size:full tile:10x10
        box label:Header gsize:10x2 text:ここにヘッダーが入ります
        box label:Sidebar gsize:3x7 text:サイドバー。メニューなど
        box label:Body gsize:7x7 text:本文。メインコンテンツ
            flex size:full direction:column
                box label:Form text:ここに入力フォームが入ります
                    フォームは部品をテンプレート化した方が良いかもしれません
                box label:Preview text:入力結果がプレビューされます
        box label:Footer gsize:10x1 text:フッターです
"""

tree = parse_indented_text(text)
stack_json = json.dumps(
    tree.to_dict(),
    ensure_ascii=False, indent=4
)
#print(stack_json)

page = None
print(json.dumps(
    ensure_ascii=False, indent=4
))

print("linking")
linked_page = None
print(json.dumps(
    linked_page.to_dict(),
    ensure_ascii=False, indent=4
))

print("parsed")
p = parser(linked_page)
parsed = p.parse()
print(json.dumps(
    parsed.to_dict(),
    ensure_ascii=False, indent=4
))
print("converted")
converted = convert(parsed)
print(json.dumps(
    converted.to_dict(),
    ensure_ascii=False, indent=4
))

print("rendered")
r = Renderer(converted)
rendered = r.render()
print(rendered)
print("done")


"""
ParsedNodeに、関数処理を追加
- パラメータのdictのキーが、関数一覧と一致してた場合
    - 関数を実行して、パラメータ変換を行う
- 一致していなかった場合
    - そのままパラメータを使う
"""
from .basic_tmpls import basic_methods

class ConvertedNode:
    def __init__(self, tmpl):
        self.tmpl = tmpl
        self.params = {}
        self.children = []

    def __repr__(self):
        return f"ConverteddNode(tmpl={self.tmpl}, params={self.params}, children={self.children})"
    
    def to_dict(self):
        return {
            "tmpl": self.tmpl,
            "params": self.params,
            "children": [child.to_dict() for child in self.children]  # 再帰的に辞書化
        }

def convert(parsedNode):

    # ConvertedNodeを生成
    convertedNode = ConvertedNode(parsedNode.tmpl)
    # パラメータのdictのキーが、関数一覧と一致してた場合
    for key in parsedNode.params.keys():
        method_name = f"{parsedNode.tmpl}_{key}"
        if method_name in basic_methods.keys():
            # 関数を実行して、params["style"]に書き足していく
            if "style" not in convertedNode.params:
                convertedNode.params["style"] = ""
            convertedNode.params["style"] += basic_methods[method_name](parsedNode.params[key])
        else:
            # 一致していなかった場合、そのままパラメータを使う
            convertedNode.params[key] = parsedNode.params[key]
    # 子ノードを再帰的に処理する
    for child in parsedNode.children:
        convertedNode.children.append(convert(child))
    return convertedNode
    


from jinja2 import Template
from . import basic_tmpls
from .tokenizer import parse_indented_text
from .parser import parser
from .converter import convert
from .renderer import Renderer

class MukuroL:
    def __init__(self, templates=None, methods=None):
        self.templates = templates or {}
        self.methods = methods or {}
        self.add_templates(basic_tmpls.basic_tmpls)
        self.add_methods(basic_tmpls.basic_methods)

    def add_templates(self, templates):
        self.templates.update(templates)

    def add_methods(self, methods):
        self.methods.update(methods)

    def generate_html(self, mukurol_text):
        tree = parse_indented_text(mukurol_text)
        page = tree.children[0]
        # レイアウトHTMLの生成
        p = parser(page)
        parsed = p.parse()
        converted = convert(parsed)
        r = Renderer(converted)
        rendered = r.render()
        lines = [line for line in rendered.splitlines() if line.strip()]
        layout =  "\n".join(lines)

        return layout

